
## Author: Davide Vespasiani 2020

library(data.table);library(magrittr);library(dplyr)
library(tidyr);library(openxlsx) ## this is for exporting tables in excel format
library(wesanderson);library(RColorBrewer); library(viridis);library(viridisLite)## these  are just for colors
library(ggthemes);library(ggplot2);library(ggpubr)
library(ggrepel)
library(readxl)


numb_threads=getDTthreads()
threads=setDTthreads(numb_threads-1)

setwd('/Users/lauracook/OneDrive - The University of Melbourne/PhD (2018-2021)/5-TFBS/4-Results/1-BioMotif/')

plot_dir='~/Desktop' 
table_dir='~/Desktop/'

input_file <- fread("motifs2.csv", sep=",", header=TRUE)

## these lines count the number of twars per condition (i.e. lost/gained/conserved) in each cluster
## this is done with data.table package (if you dont know and you'll need R my adise is to look for it, it's very nice)
## in any case when you see object=object[,colname:=something] it basically creates a column (colname) with entries corresponding on the arguments on the right
## or if you see object=object[,c('colname1,'colname2'...)], it subset the objects retaining only the columns specified
input_file_counts=copy(input_file)
input_file_counts=input_file_counts[,c('twar','gain.lost','vierstra_cluster_no.','vierstra_cluster_name')] %>% 
  unique() ## ps: if you have a twar that disrupts the motif recognised by >1 tf belonging to the same cluster calling unique here will remove this duplicate and count it only once 
input_file_counts=input_file_counts[,tfs_in_cluster_percondition:=.N,by=.(vierstra_cluster_no.,gain.lost)][ ## this means count all rows (i.e. .N) by= factor level in specified column
  ,totTFs_percondition:=.N,by=.(gain.lost) 
  ][,tfs_not_cluster_percondition:=totTFs_percondition-tfs_in_cluster_percondition] ## row-wise subtraction

## this function returns you a simplified version of the df subsetting it for the specified condition
simplify_table=function(x,condition){
  df=copy(x)[gain.lost%in%condition][
    ,c('vierstra_cluster_no.','vierstra_cluster_name','tfs_in_cluster_percondition','tfs_not_cluster_percondition')
    ] %>% unique()## this unique here simplifies the resulting table
}

input_file_counts_altered=simplify_table(input_file_counts,'Altered')
input_file_counts_lost=simplify_table(input_file_counts,'Lost')
input_file_counts_conserved=simplify_table(input_file_counts,'Conserved')

concatenate_df=function(x){
  bgkr=input_file_counts_conserved
  test=copy(x)
  ## this line is an inner join, so it returns all rows of test where there is a matching value in bkgr for rows of the two columns specified in 'on' argument
  ## i.e. it returns the clusters in the test set that are also in the bkgr set so that u can create the matrix for fisher test
  test=test[bgkr,on=c('vierstra_cluster_no.','vierstra_cluster_name'),nomatch=0]%>% 
    setnames(old=c('tfs_in_cluster_percondition','tfs_not_cluster_percondition','i.tfs_in_cluster_percondition','i.tfs_not_cluster_percondition'),
             new=c('test_in_cluster','test_not_cluster','bkgr_in_cluster','bkgr_not_cluster'))
}

input_altered_vs_conserved=concatenate_df(input_file_counts_altered)

input_gained_vs_conserved=concatenate_df(input_file_counts_gained)
input_lost_vs_conserved=concatenate_df(input_file_counts_lost)

## this function computes the fisher.exact pvalues and fdr corrects them
fisher_pvalues=function(x){
  pvals=copy(x)
  pvals=pvals[,c('vierstra_cluster_no.','vierstra_cluster_name'):=NULL]
  pvals=apply(pvals, 1, 
              function(y) {
                tbl <- matrix(as.numeric(y), ncol=2, byrow=T)
                fisher.test(tbl)$p.value
              }) ## this function applies to every row in your df. for each row it creates the 2x2 matrix and perform fisher test and returns just the p.value as vector of characters
  ## ps if you are not sure how it works/dont believe me (i would understand it) just run this:
  ## pvals=copy(your_input); pvals=pvals[,c(1:2):=NULL][1,] take first row only and then matrix(as.numeric(pvals), ncol=2, byrow=T). you'll see how it makes the 2x2 table
  
  pvalues_table=data.table(pval=pvals) ## then this converts the list of pvalues into a column (orders are maintained, so the first pval refers to the fisrt row in the input dataframe and so on)
  pvalues_table=pvalues_table[
    ,adj_p:=p.adjust(pval,method = 'fdr')][ ## this creates another column with the adj_pvalues 
      ,log10_p_adjust:=-log10(adj_p)
      ][ 
        ,significant_score:=ifelse(`adj_p`<=0.0001,'****',
                                   ifelse(`adj_p`>0.0001 &`adj_p`<=0.001,'***',
                                          ifelse(`adj_p`>0.001 & `adj_p`<=0.01,'**',
                                                 ifelse(`adj_p`>0.01 & `adj_p`<=0.05,'*',' '))))
        ] ## this is just for quick visualisation/subsecting of only the significant ones  
  
  return(pvalues_table)
}

input_altered_vs_conserved_pvals=fisher_pvalues(input_altered_vs_conserved)

input_gained_vs_conserved_pvals=fisher_pvalues(input_gained_vs_conserved)
input_lost_vs_conserved_pvals=fisher_pvalues(input_lost_vs_conserved)

## now it creates final df with all columns you need 
altered_vs_conserved=cbind(input_altered_vs_conserved,input_altered_vs_conserved_pvals)

gained_vs_conserved=cbind(input_gained_vs_conserved,input_gained_vs_conserved_pvals)
lost_vs_conserved=cbind(input_lost_vs_conserved,input_lost_vs_conserved_pvals)

## this computes fold enrichment of the number of twars lost/gained for each cluster over that of conserved ones for the same cluster
## ps this is the way i have been calculating the fold enrichmnent(i.e. the ratio of your test set/bkgr set in the cluster over the mean of this ratio across all clusters) but you could change it 
fold_enrichment=function(x,group){
  df=copy(x)
  df=df[
    ,group:=group  ## ps: i have created here a column that takes values specified in group argument (i.e. gained/lost) because this allows me at the end to combine the 2 df and use a single function to plot them together (in separate facets) -see below
    ][,ratio_twar_lostgain_cluster:=test_in_cluster/bkgr_in_cluster
      ][,mean_ratio:=mean(ratio_twar_lostgain_cluster)
        ][,log2_fold_enrichment:=log2(ratio_twar_lostgain_cluster/mean_ratio)
          ][,log10_numb_twars_incluster:=log10(test_in_cluster)
            ][,c('vierstra_cluster_no.','vierstra_cluster_name','log2_fold_enrichment','pval','adj_p',
                 'significant_score','log10_p_adjust','log10_numb_twars_incluster','group')
              ]%>% setorderv('log10_p_adjust',-1)
}

altered_vs_conserved_final=fold_enrichment(altered_vs_conserved,'Altered')

gained_vs_conserved_final=fold_enrichment(gained_vs_conserved,'Gained')
lost_vs_conserved_final=fold_enrichment(lost_vs_conserved,'Lost')

## here i make a single dt
final_table=rbind(gained_vs_conserved_final,lost_vs_conserved_final)


final_table=rbind(lost_vs_conserved_final)


## make volcano plot
tf_enrich_plot=function(x){
  df=copy(x)[,'Log10 total TFs per Cluster':=log10_numb_twars_incluster]
  gradient=scale_colour_viridis(aes(`Log10 total TFs per cluster`),option="inferno",discrete = F)
  #text=ifelse(!df$significant_score%in%' ',df$vierstra_cluster_name,'') ## this wont report in the plot the name of the clusters that are not significantly enriched/depleted
  
  ggplot(df,aes(x=log2_fold_enrichment,log10_p_adjust,label = text, col=log10_numb_twars_incluster))+
    geom_point(size=2)+
    geom_vline(xintercept=0, linetype="dashed", color = "black",size=0.2)+
    geom_text_repel(size = 5,color='black',
                    box.padding = unit(0.5, "lines"),
                    point.padding = unit(0.5, "lines")
    )+
    gradient+
    xlab('\n Log2 fold enrichment \n')+
    ylab('-Log10 (P)')+
    xlim(-4,4)+
    facet_wrap(group~.,ncol = 2)+ ## so this is what i was saying above: the group column allows now to split the plots into 2 facets 
    theme(strip.text.x = element_text(),
          strip.text.y = element_text(hjust = 0.5),
          strip.background = element_rect(color = 'black', linetype = 'solid'),
          strip.background.y = element_blank(),
          strip.background.x =element_blank(),
          panel.spacing=unit(1, "lines"),
          panel.background =element_rect(fill = 'white', colour = 'black',size=1),
          panel.grid.minor = element_blank(),
          panel.grid.major = element_blank(),
          legend.position = "bottom",
          legend.key = element_rect(fill = "white", colour = "black"),
          axis.line = element_blank())
}


pdf(paste(plot_dir,'TFBS_enrichment_fisher_test_volcano.pdf',sep=''),width = 10,height=6)
tf_enrich_plot(final_table)
dev.off()

## This code below produces an excel file with gained/lost enrichment results in two ≠ sheets 

excel_table=copy(final_table)
excel_table=split(excel_table,as.factor(excel_table$group)) %>%lapply(function(y)y=y[
  ,c('group','vierstra_cluster_no.','vierstra_cluster_name','log2_fold_enrichment','log10_numb_twars_incluster','pval','adj_p','log10_p_adjust','significant_score')]%>%
    unique())
names(excel_table)=c('altered') ## name of the sheets

write.xlsx(excel_table,paste(table_dir,'Table_twars_fisher_pvalues.xlsx',sep=''))